close all
clear 
clc

% Clear all variables, close all figures, and clear command window
clear; close all; clc;

%% Input Parameters
% Angular frequency [rad/s]
w = 360;
% Length of rod [m]
LX = 1; LY = 1;
% Wave number
k = 30;

% Imaginary unit
i = sqrt(-1);
lambda = 2 * pi / k;
h = lambda / 10;

% Spatial domain [m]
x = 0:h:LX;
y = 0:h:LY;
[X, Y] = meshgrid(x, y);

% Number of grid points
N = floor(LX / h) - 1;
r = (N + 2)^2 - 1;
% Define K2 matrix based on condition
K = zeros(N+2,N+2);
condition = (X - 0.5).^2 + (Y - 0.5).^2 < 0.01;
K(condition) = 100;  % Inside domain
K(~condition) = k;    % Outside domain

% Coefficient for forward problem
  a = -1 / (h^2);
  c = complex(3 / (h^2) - k^2, k / h);
  d = complex(2 / (h^2) - k^2, sqrt(2) * (k / h));
% Number of eigenfunctions
M = 100;
% Mode number for graphical outputs
mX = 4;

%% Source Term Calculation using construct_source_term
[F, f] = construct_source_term(X, Y, N);

%% Construct Matrix H using the construct_H function
H = construct_H(K, N, h, k);

%% Solve for y_sol
y_sol = H \ F;
Y_sol = reshape(y_sol, N+2, N+2);
%% Background k_0 via Solving the Elliptic Problem
K0=zeros((N+2)^2,1);
for j=1:(N+2)^2
    K0(j)=k;
end
%% Define Projection Matrix and Observation Data using define_observation_data
 p = random_projection(N);
    % Project the solution onto the observation space
    pmatrix = p * y_sol;
    % Add Gaussian noise to the projected data
    epsilonr = (0.005 / sqrt(2)) * std(pmatrix) * randn(r, 1);
    epsiloni = (0.005 / sqrt(2)) * std(pmatrix) * randn(r, 1);
    epsilon = complex(epsilonr, epsiloni);
    % Noisy observed data
    yobs = pmatrix + epsilon;

%% Finding the Eigenfunctions basis for first adaptation 
Phi_1 =first_eigenfunctions(a, N, M, mX);

%% Initialize parameters for the Gauss-Newton method
beta = zeros(M, 1);  % Initial guess for beta (zero vector)
% Armijo parameters
alpha =1;          % initial step size for Armijo line search
beta_armijo = 0.1;  % reduction factor for Armijo line search
tau =0.9;          % Armijo line search parameter
tol = 1e-3; % tolerance for convergence
maxIter = 10; % maximum number of iterations
% Gauss-Newton iteration
for iter = 1:maxIter
  %Step 1: Compute the first K of inverse problem
    K1 = K0 + Phi_1 * beta;
    K1_re = reshape(K1, N + 2, N + 2);
    K1_re = K1_re';
    % Step 2: Compute H[K1] and dH[K1]_dK using the external function
    H1 = construct_H(K1_re, N, h, k);  % Call to my external function to construct H
    dH1_dk = construct_dH_dk(K1_re, N);
    H1_inv=H1\ eye(size(H1));
    P1=p *H1_inv;
   % Step 3: Compute residual r1
    r1 =  P1*F-  yobs ;
   % Step 4: compute Jacobian of the residual
    J=-P1*dH1_dk*H1_inv*F;
   % Step 5:Solve for  delta_beta
    delta_beta=-(J' * J) \ (J' * r1);
   % Step 6:define descent direction 
    dis_direction=(J' * r1)'*delta_beta;
    normr=norm(r1,2)^2;
   % Step 7:Solve for  new beta
    beta_1=beta+delta_beta;
   %Step 8: Compute new K2 
    K2 = K0 + Phi_1 * beta_1;
    K2_re = reshape(K2, N + 2, N + 2);
    K2_re = K2_re';
   % Step 9: Compute H[K2]  using the external function
    H2 = construct_H(K2_re, N, h, k);  % Call to my external function to construct H
    H2_inv=H2\ eye(size(H2));
    P2=p *H2_inv;
   % Step 10: Compute residual r2
    r2 =  P2*F-  yobs ;
   % Step 11:compute Armigo line search
   step_size=alpha;
   count=0;
   while norm(r2, 2)^2>normr-beta_armijo*step_size*dis_direction
      step_size = tau* step_size;
      beta_1=beta+step_size* delta_beta;
       %solve for new K
      K2=K0+Phi_1*beta_1;
      K2_re=reshape(K2,N+2,N+2);
      K2_re=K2_re';
      % obtain new H
      H2 = construct_H(K2_re, N, h, k);  % Call to my external function to construct H
      H2_inv=H2 \ eye(size(H2));
      P2=p *H2_inv;
      % Compute residual r
       r2 =  P2*F-  yobs ;
      newnorm=norm(r2,2);
      fprintf('newnorm%d=%4.7f,, step_size = %.6f\n',count,newnorm, step_size)
      count=count+1;
   end 
   % Step 11:check for convergence
   if norm(r2,2)<tol
       break
   end
   % Step 12:update for beta
   beta=beta_1;
end
 %  finding the scatter term k=k_0_new+<beta,eignFN>
 K_first=K0+(Phi_1*beta);
 K_First_re=reshape(K_first,(N+2),(N+2));
 K_First=K_First_re';
 figure(3)
 imagesc(real(K_First))
return
 %% AEI
% AEI STEP2: obtain backgrond K0_1
K0_1 = compute_K0_1(K_First, N, h, k);
figure(4)
imagesc(real(K0_1))
% AEI STEP2:compute new eigenfunctions basis
Phi_2 = compute_eigenfunctions_AEI(K_First, N, h, M);
 %   V_2 = zeros((N+2)^2,1);
 %   V_2(1:((N+2)^2)) = Phi_2(:,mX);
 %   V_2_re=reshape(V_2,(N+2),(N+2));
 %   figure(5)
 % imagesc(real(V_2_re))
return
% Gauss Newton method 
beta1 = zeros(M, 1);  % Initial guess for beta1 (zero vector)
% Armijo parameters
alpha =1;          % initial step size for Armijo line search
beta_armijo = 0.1;  % reduction factor for Armijo line search
tau =0.6;          % Armijo line search parameter
tol = 1e-3; % tolerance for convergence
maxIter = 1; % maximum number of iterations
% Gauss-Newton iteration
for iter = 1:maxIter
  %Step 1: Compute the first K of inverse problem
    K2 = K0_1 + Phi_2 * beta;
    K2_re = reshape(K2, N + 2, N + 2);
    K2_re = K2_re';
    % Step 2: Compute H[K2] and dH[K2]_dK using the external function
    H2 = construct_H(K2_re, N, h, k);  % Call to my external function to construct H
    dH2_dk = construct_dH_dk(K2_re, N);
    H2_inv=H2\ eye(size(H2));
    P2=p *H2_inv;
   % Step 3: Compute residual r1
    r2 =  P2*F-  yobs ;
   % Step 4: compute Jacobian of the residual
   J1=-P2*dH2_dk*H2_inv*F;
   % Step 5:Solve for  delta_beta
   delta_beta1=-(J1' * J1) \ (J1' * r2);
   % Step 6:define descent direction 
   dis_direction1=(J1' * r2)'*delta_beta1;
   normr1=norm(r2,2)^2;
   % Step 7:compute Armigo line search
   step_size=alpha;
   count=0;
   while norm(r2, 2)^2>normr1-beta_armijo*step_size*dis_direction1
      step_size = tau* step_size;
      beta_2=beta1+step_size* delta_beta1;
       %solve for new K
      K2_new=K0+Phi_1*beta_1;
      K2_new_re=reshape(K2_new,N+2,N+2);
      K2_new_re=K2_new_re';
      % obtain new H
      H2_new = construct_H(K2_new_re, N, h, k);  % Call to my external function to construct H
      H2_new_inv=H2_new \ eye(size(H2_new));
      P2_new=p *H2_new_inv;
      % Compute residual r
       r2_new =  P2_new*F-  yobs ;
      newnorm1=norm(r2_new,2);
      fprintf('newnorm%d=%4.7f,, step_size = %.6f\n',count,newnorm1, step_size)
      r2=r2_new;
      count=count+1;
   end 
   % beta=beta_1;
end

 %  finding the scatter term k=k_0_new+<beta,eignFN>
 K_second=K0_1+(Phi_2*beta1);
 K_Second=reshape(K_second,(N+2),(N+2));

   figure(6)
  imagesc(real(K_Second))



