function [yobs, pmatrix] = define_data(y_sol, N)
    % Number of elements in the reshaped solution
    r = (N+2)^2 - 1;

    % Random sample from the elements of the reshaped matrix
    D = randsample(1:(N+2)^2, r);

    % Define s as half of r, rounded down
    s = floor(0.5 * r);

    % Randomly select s elements
    n = randsample(min(D):max(D), s);

    % Initialize q vector
    q = zeros(1, (N+2)^2);

    % Set the selected positions in q to 1
    for i = 1:s
        q(n(i)) = 1; 
    end

    % Initialize projection matrix p as an identity matrix
    p = eye(r, (N+2)^2);

    % Set certain diagonal entries to 0 based on q
    for i = 1:r
        if p(i, i) ~= q(i)
            p(i, i) = 0;
        end
    end

    % Project the solution onto the observation space
    pmatrix = p * y_sol;

    % Add Gaussian noise to the projected data
    epsilonr = (0.005 / sqrt(2)) * std(pmatrix) * randn(r, 1);
    epsiloni = (0.005 / sqrt(2)) * std(pmatrix) * randn(r, 1);
    epsilon = complex(epsilonr, epsiloni);

    % Noisy observation data
    yobs = pmatrix + epsilon;
end
